# Dat je fajl "car_price_dataset.csv" sa informacijama
# o cenama automobila. Nacrtati sledece grafike:
# 1. Horizontalni stubicasti grafik (bar chart) popularnosti
#    proizvodjaca.
# 2. Za svakog proizvodjaca automobila graficki prikazati (pie chart)
#    popularnost modela koji su u ponudi.
# 3. Za svaku godinu koja postoji u istrazivanju prikazati prosecnu
#    cenu, maksimalnu cenu i minimalnu cenu vozila tog godista.
# 4. Za svaku godinu koja postoji u istrazivanju prikazati prosecnu
#    kilometrazu vozila tog godista.
# 5. Za svaku godinu koja postoji u istrazivanju odrediti popularnost
#    razlicitih vrsta pogona.

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from collections import Counter

def popularnost_proizvodjaca(data):
    
    proizvodjaci = data["Brand"]
    proizvodjaci_cnt = Counter()
    for proizvodjac in proizvodjaci:
        proizvodjaci_cnt.update([proizvodjac])
        
    proizvodjac = []
    popularnost = []
    for (el,cnt) in proizvodjaci_cnt.most_common():
        proizvodjac.append(el)
        popularnost.append(cnt)

    proizvodjac.reverse()
    popularnost.reverse()

    return (proizvodjac, popularnost)

def popularnost_modela(data, proizvodjac):
    
    modeli_cnt = Counter()
    filt = data["Brand"] == proizvodjac
    modeli = data.loc[filt, "Model"]
    for model in modeli:
        modeli_cnt.update([model])
        
    model = []
    popularnost = []
    for (m, cnt) in modeli_cnt.most_common():
        model.append(m)
        popularnost.append(cnt)
        
    return (model, popularnost)

def cene_vozila_po_godinama(data):
    
    godine = data.groupby("Year")
    statistika_cena = godine["Price"].agg(["min", "max", "mean"])
    
    return statistika_cena

def km_vozila_po_godinama(data):
    
    godine = data.groupby("Year")
    statistika_km = godine["Mileage"].agg(["min", "max", "mean"])
    
    return statistika_km

def popularnost_pogona(data, godina):
    
    pogoni_cnt = Counter()
    filt = data["Year"] == godina
    pogoni = data.loc[filt, "Fuel_Type"]
    for pogon in pogoni:
        pogoni_cnt.update([pogon])
        
    pogon = []
    popularnost = []
    for (m, cnt) in pogoni_cnt.most_common():
        pogon.append(m)
        popularnost.append(cnt)
    
    sortirani = sorted(zip(pogon, popularnost), key = lambda x: x[0])
    return (godina, sortirani)

if __name__ == "__main__":

    data = pd.read_csv("car_price_dataset.csv")
    (proizvodjac, popularnost) = popularnost_proizvodjaca(data)

    plt.figure(1)
    plt.barh(proizvodjac, popularnost)

    plt.xlabel("Popularnost")
    plt.ylabel("Proizvodjac")
    plt.title("Popularnost proizvodjaca automobila")

    plt.grid()
    plt.tight_layout()
    
    fig, axs = plt.subplots(2, 5, figsize=(10,6), layout='constrained')
    k = 0
    for pr in proizvodjac:
        (model, popularnost) = popularnost_modela(data, pr)

        i = int(k // 5)
        j = int(k % 5)
        
        axs[i][j].pie(popularnost,labels=model, wedgeprops={'edgecolor': 'black'},
                      startangle=90, autopct='%1.1f%%')
        axs[i][j].set_title(pr)
        axs[i][j].legend(loc='upper left', bbox_to_anchor=(0, 0))
        fig.align_labels()
        
        k = k + 1
    fig.suptitle("Popularnost modela")
    
    cene = cene_vozila_po_godinama(data)
    plt.figure(3)
    plt.plot(cene.index, cene["mean"], label="Prosecna cena")
    plt.plot(cene.index, cene["min"], label="Minimalna cena")
    plt.plot(cene.index, cene["max"], label="Maksimalna cena")

    plt.xlabel("Godina proizvodnje")
    plt.ylabel("Cene")
    plt.title("Statstika cena vozila po godinama")
    plt.legend()

    plt.grid()
    plt.tight_layout()
    
    km = km_vozila_po_godinama(data)
    plt.figure(4)
    plt.plot(km.index, km["mean"]*1.6, label="Prosecna km")
    plt.plot(km.index, km["min"]*1.6, label="Minimalna km")
    plt.plot(km.index, km["max"]*1.6, label="Maksimalna km")

    plt.xlabel("Godina proizvodnje")
    plt.ylabel("Kilometraza")
    plt.title("Statstika km vozila po godinama")
    plt.legend()

    plt.grid()
    plt.tight_layout()
    
    godine = km.index
    pogoni = []
    for godina in godine:
        pogon = popularnost_pogona(data, godina)
        pogoni.append(pogon)
    godine = []
    dizel = []
    elektricni = []
    hibrid = []
    benzin = []
    for x in pogoni:
        godine.append(x[0])
        dizel.append(x[1][0][1])
        elektricni.append(x[1][1][1])
        hibrid.append(x[1][2][1])
        benzin.append(x[1][3][1])
    x_indexes = np.arange(len(godine))
    
    width = 0.2
    
    plt.figure(5)
    plt.bar(x_indexes - 3*width/2, dizel, width = width, label="dizel")
    plt.bar(x_indexes - width/2, elektricni, width = width, label="elektricni")
    plt.bar(x_indexes + width/2, hibrid, width = width, label="hibrid")
    plt.bar(x_indexes + 3*width/2, benzin, width = width, label="benzin")
    
    plt.xticks(ticks=x_indexes, labels=godine)
    
    plt.ylabel("Popularnost pogona")
    plt.xlabel("Godine proizvodnje")
    plt.title("Popularnost pogona vozila po godinama")
    plt.legend()

    plt.grid()
    plt.tight_layout()
    
    plt.show()
    
    